/*
Code written by: Dean Lunz (aka Crated by: X)
Web: http://www.createdbyx.com/ 
 */
#region Using Statements
using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Storage;
#endregion

namespace GunsABlazin
{
    /// <summary>
    /// This is a game component that implements IUpdateable.
    /// </summary>
    public partial class Actor : Microsoft.Xna.Framework.DrawableGameComponent
    {

        private Vector2 pos = Vector2.Zero;
        private float dir = 0;
        private float speed = 1;            
        private float health = 100;
        private FrameSequence frames;
        private float maxSpeed = 100;
        private SpriteBatch sprite;

        public enum MoveDirection
        {
            None = 0,
            Forward = 1,
            Backward = 2,
            Left = 4,
            Right = 8
        }

        public void Move(MoveDirection d )
        {
            if (d == MoveDirection.None) return;

            Vector2 tmp = Vector2.Zero;
            //if (d & 1)
            //{
            //    tmp.X += (float)Math.Cos(dir - MathHelper.PiOver2) * speed;
            //    tmp.Y += (float)Math.Sin(dir - MathHelper.PiOver2) * speed;
            //}
                
            pos += tmp;
        }

        public void MoveForward(GameTime gameTime)
        {
            pos.X += (float)Math.Cos(dir - MathHelper.PiOver2) * (speed * gameTime.ElapsedGameTime.Milliseconds);
            pos.Y += (float)Math.Sin(dir - MathHelper.PiOver2) * (speed * gameTime.ElapsedGameTime.Milliseconds);
        }

        public void MoveBackward(GameTime gameTime)
        {
            pos.X += (float)Math.Cos(dir + MathHelper.PiOver2) * (speed * gameTime.ElapsedGameTime.Milliseconds);
            pos.Y += (float)Math.Sin(dir + MathHelper.PiOver2) * (speed * gameTime.ElapsedGameTime.Milliseconds);
        }

        public void StrafeLeft(GameTime gameTime)
        {
            pos.X += (float)Math.Cos(dir - MathHelper.Pi) * (speed * gameTime.ElapsedGameTime.Milliseconds);
            pos.Y += (float)Math.Sin(dir - MathHelper.Pi) * (speed * gameTime.ElapsedGameTime.Milliseconds);
        }

        public void StrafeRight(GameTime gameTime)
        {
            pos.X += (float)Math.Cos(dir) * (speed * gameTime.ElapsedGameTime.Milliseconds);
            pos.Y += (float)Math.Sin(dir) * (speed * gameTime.ElapsedGameTime.Milliseconds);
        }

        public float MaxSpeed
        {
            get { return maxSpeed; }
            set { maxSpeed = value; }
        }

        public FrameSequence Frames
        {
            get { return frames; }
            set 
            {
                if (value == null) throw new ArgumentNullException("value");
                if (frames == value) return;
                frames = value;
                frames.Reset();
            }
        }

        public float Health
        {
            get { return health; }
            set { health = value; }
        }

        public float Speed
        {
            get { return speed; }
            set
            {
                speed = value;
                if (speed > maxSpeed) speed = maxSpeed;
            }
        }

        public float Direction
        {
            get { return dir; }
            set { dir = value; }
        }

        public Vector2 Position
        {
            get { return pos; }
            set { pos = value; }
        }

        public Actor(Game game)
            : base(game)
        {
            // TODO: Construct any child components here
            frames = new FrameSequence(game);
            this.Game.Components.Add(frames);
        }


        /// <summary>
        /// Allows the game component to perform any initialization it needs to before starting
        /// to run.  This is where it can query for any required services and load content.
        /// </summary>
        public override void Initialize()
        {
            // TODO: Add your initialization code here

            base.Initialize();
            sprite = new SpriteBatch(this.GraphicsDevice);
        }


        /// <summary>
        /// Allows the game component to update itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        public override void Update(GameTime gameTime)
        {
            // TODO: Add your update code here

            base.Update(gameTime);
        }

        public override void Draw(GameTime gameTime)
        {
            // TODO: Add your Draw code here
            if (frames.Count == 0) return;
            if (frames.CurrentFrame == -1) return;

            FrameSequenceItem f = frames.GetCurrentFrame();
            if (f == null) return;

            sprite.Begin();
            sprite.Draw(f.Texture, new Vector2(pos.X, pos.Y), null, Color.White, dir, f.Center, 1, SpriteEffects.None, 0);
            sprite.End();

            base.Draw(gameTime);
        }
    }
}


