/*
Code written by: Dean Lunz (aka Crated by: X)
Web: http://www.createdbyx.com/ 
 */
//#define Win32

#region Using Statements
using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Storage;
#endregion

namespace GunsABlazin
{
    /// <summary>
    /// This is the main type for your game
    /// </summary>
    public class Game1 : Microsoft.Xna.Framework.Game
    {
        GraphicsDeviceManager graphics;
        ContentManager content;
        Player p1;
        FrameSequenceManager fsmanager;
        BulletManager bulletManager;
        XNAFont font;
        FPSCounter fps;
        Collections.TextureDictionary textures;
        SpriteBatch sb;
        float targetMoveSpeed = 0.5f;
        GamePadState prevstate;

#if Win32
        private int prevMouseScrollValue;
#endif

        public FrameSequenceManager FSManager
        {
            get { return fsmanager; }
        }

        public Game1()
        {
            graphics = new GraphicsDeviceManager(this);
            content = new ContentManager(Services);
        }


        /// <summary>
        /// Allows the game to perform any initialization it needs to before starting to run.
        /// This is where it can query for any required services and load any non-graphic
        /// related content.  Calling base.Initialize will enumerate through any components
        /// and initialize them as well.
        /// </summary>
        protected override void Initialize()
        {
            // TODO: Add your initialization logic here
            sb = new SpriteBatch(this.graphics.GraphicsDevice);
            textures = new Collections.TextureDictionary();
            fsmanager = new FrameSequenceManager();
            bulletManager = new BulletManager(this);
            bulletManager.BulletReferences.Add("Basic", new BasicBullet());
            bulletManager.BulletReferences.Add("Grenade", new BasicGrenade());
            this.Components.Add(bulletManager);

            p1 = new Player();
            p1.Actor = new Actor(this);
            p1.GamePad = PlayerIndex.One;
            p1.Actor.Position = new Vector2(this.graphics.GraphicsDevice.Viewport.Width / 2, this.graphics.GraphicsDevice.Viewport.Height / 2);
            p1.Target = p1.Actor.Position + (Vector2.UnitY * 100);
            this.Components.Add(p1.Actor);


            p1.Weapons.Add(new Weapons.StandardGun(bulletManager));
            p1.Weapons.Add(new Weapons.Grenade(bulletManager));
            p1.CurrentWeapon = 0;
            p1.GetCurrentWeapon().Bullet = "Basic";
            p1.Weapons[1].Bullet = "Grenade";
            p1.BulletSpawnOffsett = new Vector2(13, -23);

            fps = new FPSCounter(this);
            this.Components.Add(fps);
            base.Initialize();
        }

        private void LoadBullets()
        {
            Bullet b = new Bullet();

            // create standard bullet sequence
            FrameSequence fs = new FrameSequence(this);
            FrameSequenceItem Item = new FrameSequenceItem();
            Item.DisplayTime = TimeSpan.FromMilliseconds(1);
            Item.Texture = content.Load<Texture2D>(@".\Images\StandardBullet");
            Item.Center = new Vector2(10, 2);
            fs.Add(Item);

            // add frame sequence to game components
            this.Components.Add(fs);
            // add a standing sequence
            bulletManager.Sequences.Add("Standard", fs);
            bulletManager.BulletReferences["Basic"].BulletSequence = fs;

            // add grenade
            fs = new FrameSequence(this);
            Item = new FrameSequenceItem();
            Item.DisplayTime = TimeSpan.FromMilliseconds(1);
            Item.Texture = content.Load<Texture2D>(@".\Images\Grenade");
            Item.Center = new Vector2(6, 7);
            fs.Add(Item);
            this.Components.Add(fs);
            bulletManager.Sequences.Add("Grenade", fs);
            bulletManager.BulletReferences["Grenade"].BulletSequence = fs;
        }


        /// <summary>
        /// Load your graphics content.  If loadAllContent is true, you should
        /// load content from both ResourceManagementMode pools.  Otherwise, just
        /// load ResourceManagementMode.Manual content.
        /// </summary>
        /// <param name="loadAllContent">Which type of content to load.</param>
        protected override void LoadGraphicsContent(bool loadAllContent)
        {
            if (loadAllContent)
            {

                // load target textures
                textures.Add("Target1", content.Load<Texture2D>(@".\Images\Target1"));
                textures.Add("Target2", content.Load<Texture2D>(@".\Images\Target2"));

                // TODO: Load any ResourceManagementMode.Automatic content
                // create new frame sequence to store actor frames
                FrameSequence fs = new FrameSequence(this);
                for (int idx = 120; idx <= 123; idx++)
                {
                    FrameSequenceItem Item;
                    Item = new FrameSequenceItem();
                    Item.DisplayTime = TimeSpan.FromMilliseconds(100);
                    Item.Texture = content.Load<Texture2D>(@".\Images\5-" + idx.ToString());
                    Item.Center = new Vector2(14, 26);
                    fs.Add(Item);
                }

                // add walking sequence
                this.Components.Add(fs);
                fsmanager.Sequences.Add("walking", fs);

                // create standing sequence
                fs = new FrameSequence(this);
                fs.Add(fsmanager.Sequences["walking"].GetFrame(1));
                this.Components.Add(fs);
                // add a standing sequence
                fsmanager.Sequences.Add("standing", fs);
                // assign standing sequence to player's actor reference
                p1.Actor.Frames = fs;

                // load bullets
                LoadBullets();

                // load and setup font
                font = XNAFont.LoadFont(this.graphics.GraphicsDevice, @".\Fonts\Font.dds.xml", string.Empty);
                font.Texture = content.Load<Texture2D>(@".\Fonts\Font");

                // assign weapon target textures
                p1.Weapons[0].TargetTexture = textures["Target1"];
                p1.Weapons[1].TargetTexture = textures["Target2"];

                // set player speed
                p1.Actor.Speed = 0.2f;
            }

            // TODO: Load any ResourceManagementMode.Manual content
        }


        /// <summary>
        /// Unload your graphics content.  If unloadAllContent is true, you should
        /// unload content from both ResourceManagementMode pools.  Otherwise, just
        /// unload ResourceManagementMode.Manual content.  Manual content will get
        /// Disposed by the GraphicsDevice during a Reset.
        /// </summary>
        /// <param name="unloadAllContent">Which type of content to unload.</param>
        protected override void UnloadGraphicsContent(bool unloadAllContent)
        {
            if (unloadAllContent == true)
            {
                content.Unload();
                font.Dispose();
                textures.Clear();
                fsmanager.Sequences.Clear();
                bulletManager.Sequences.Clear();
            }
        }


        /// <summary>
        /// Allows the game to run logic such as updating the world,
        /// checking for collisions, gathering input and playing audio.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Update(GameTime gameTime)
        {

            UpdateInputForPlayer(p1, gameTime);

            base.Update(gameTime);
        }

        private void UpdateInputForPlayer(Player player, GameTime gameTime)
        {
#if Win32
            HandleMouseInput(player, gameTime);
            HandleKeyboardInput(player, false, gameTime);
#else
                GamePadState state = GamePad.GetState(player.GamePad, GamePadDeadZone.Circular);

            // chis if game pad is connected and if it is use it
            if (state.IsConnected == true)
            {
                // check if quitting
                if (state.Buttons.Back == ButtonState.Pressed)
                {
                    this.Exit();
                    return;
                }

                // check if toggling fullscreen
                if (state.Buttons.Start == ButtonState.Released &&
                    prevstate.Buttons.Start == ButtonState.Pressed)
                {
                    this.graphics.ToggleFullScreen();
                }

                // move player
                Vector2 v = new Vector2(state.ThumbSticks.Left.X, -state.ThumbSticks.Left.Y) * (player.Actor.Speed * gameTime.ElapsedGameTime.Milliseconds);
                player.Actor.Position += v;

                if (v == Vector2.Zero)
                {
                    player.Actor.Frames = fsmanager.Sequences["standing"];
                }
                else
                {
                    player.Actor.Frames = fsmanager.Sequences["walking"];
                }

                // set the actors direction
                if (state.ThumbSticks.Right != Vector2.Zero)
                {
                    Vector2 vec = state.ThumbSticks.Right;
                    vec.Y = -vec.Y;
                    //  vec.Normalize();
                    player.Target = player.Target + (vec * (targetMoveSpeed * gameTime.ElapsedGameTime.Milliseconds));

                    vec = player.Target;
                    if (vec.X < 0) vec.X = 0;
                    if (vec.Y < 0) vec.Y = 0;
                    if (vec.X > this.graphics.GraphicsDevice.Viewport.Width) vec.X = this.graphics.GraphicsDevice.Viewport.Width;
                    if (vec.Y > this.graphics.GraphicsDevice.Viewport.Height) vec.Y = this.graphics.GraphicsDevice.Viewport.Height;
                    player.Target = vec;
                }
                
                // ensure player is facing towards there target
                v = player.Actor.Position;// -player.BulletSpawnOffsett;
                float dir = Helpers.GetRotationInRadians(v.X, v.Y, player.Target.X, player.Target.Y);
                player.Actor.Direction = dir + MathHelper.ToRadians(90);

                if (state.Buttons.RightShoulder == ButtonState.Pressed)
                {
                    ShootPlayerGun(player, gameTime);
                }

                if (state.Buttons.LeftShoulder == ButtonState.Released &&
                    prevstate.Buttons.LeftShoulder == ButtonState.Pressed)
                {
                    int tmp = player.CurrentWeapon;
                    player.CurrentWeapon = 1;
                    ShootPlayerGun(player, gameTime);
                    player.CurrentWeapon = tmp;
                }

                if (state.Triggers.Left == 0 && prevstate.Triggers.Left != 0)
                {
                    player.PrevWeapon();
                }

                if (state.Triggers.Right == 0 && prevstate.Triggers.Right != 0)
                {
                    player.NextWeapon();
                }


            }
           
            // store state
            prevstate = state;

#endif
        }

#if Win32
        private void HandleMouseInput(Player player, GameTime gameTime)
        {

            Actor act = player.Actor;
            MouseState mouse = Mouse.GetState();
            player.Target = new Vector2(mouse.X, mouse.Y);
            float dir = Helpers.GetRotationInRadians(act.Position.X, act.Position.Y, player.Target.X, player.Target.Y);
            act.Direction = dir + MathHelper.ToRadians(90);

            // check if player is trying to shoot using the mouse
            if (mouse.LeftButton == ButtonState.Pressed)
            {
                ShootPlayerGun(player, gameTime);
            }

            if (mouse.ScrollWheelValue > prevMouseScrollValue)
            {
                player.NextWeapon();
                prevMouseScrollValue = mouse.ScrollWheelValue;
            }
            else if (mouse.ScrollWheelValue < prevMouseScrollValue)
            {
                player.PrevWeapon();
                prevMouseScrollValue = mouse.ScrollWheelValue;
            }
        }

        private void HandleKeyboardInput(Player player, bool setDirection, GameTime gameTime)
        {
            Actor act = player.Actor;
            KeyboardState kb = Keyboard.GetState();
            if (kb.IsKeyDown(Keys.W) && kb.IsKeyDown(Keys.D))
            {
                // move player up and to the right
                act.Frames = fsmanager.Sequences["walking"];
                if (setDirection) act.Direction = MathHelper.ToRadians(45);
                //act.Position += (Vector3.Down + Vector3.Right) * act.Speed;
                act.StrafeRight(gameTime);
                act.MoveForward(gameTime);

            }
            else if (kb.IsKeyDown(Keys.W) && kb.IsKeyDown(Keys.A))
            {
                // move player up and to the left
                act.Frames = fsmanager.Sequences["walking"];
                if (setDirection) act.Direction = MathHelper.ToRadians(315);
                //act.Position += (Vector3.Down + Vector3.Left) * act.Speed;
                act.StrafeLeft(gameTime);
                act.MoveForward(gameTime);

            }
            else if (kb.IsKeyDown(Keys.S) && kb.IsKeyDown(Keys.A))
            {
                // move player down and to the left
                act.Frames = fsmanager.Sequences["walking"];
                if (setDirection) act.Direction = MathHelper.ToRadians(225);
                //act.Position += (Vector3.Up + Vector3.Left) * act.Speed;
                act.StrafeLeft(gameTime);
                act.MoveBackward(gameTime);

            }
            else if (kb.IsKeyDown(Keys.S) && kb.IsKeyDown(Keys.D))
            {
                // move player down and to the right
                act.Frames = fsmanager.Sequences["walking"];
                if (setDirection) act.Direction = MathHelper.ToRadians(135);
                //act.Position += (Vector3.Up + Vector3.Right) * act.Speed;
                act.StrafeRight(gameTime);
                act.MoveBackward(gameTime);

            }
            else if (kb.IsKeyDown(Keys.W))
            {
                // move player up
                act.Frames = fsmanager.Sequences["walking"];
                if (setDirection) act.Direction = MathHelper.ToRadians(0);
                //act.Position += Vector3.Down * act.Speed;
                act.MoveForward(gameTime);

            }
            else if (kb.IsKeyDown(Keys.A))
            {
                // move player left
                act.Frames = fsmanager.Sequences["walking"];
                if (setDirection) act.Direction = MathHelper.ToRadians(270);
                //act.Position += Vector3.Left * act.Speed;
                act.StrafeLeft(gameTime);

            }
            else if (kb.IsKeyDown(Keys.S))
            {
                // move player down
                act.Frames = fsmanager.Sequences["walking"];
                if (setDirection) act.Direction = MathHelper.ToRadians(180);
                //act.Position += Vector3.Up * act.Speed;
                act.MoveBackward(gameTime);

            }
            else if (kb.IsKeyDown(Keys.D))
            {
                // move player right
                act.Frames = fsmanager.Sequences["walking"];
                if (setDirection) act.Direction = MathHelper.ToRadians(90);
                //act.Position += Vector3.Right * act.Speed;
                act.StrafeRight(gameTime);

            }
            else
            {
                act.Frames = fsmanager.Sequences["standing"];
            }

            // check if user wants to quit
            if (kb.IsKeyDown(Keys.Escape))
            {
                this.Exit();
            }

            // check if user is switching weapons
            if (kb.IsKeyDown(Keys.D1) && player.Weapons.Count > 0)
            {
                player.CurrentWeapon = 0;
            }

            // check if user is switching weapons
            if (kb.IsKeyDown(Keys.D2) && player.Weapons.Count > 1)
            {
                player.CurrentWeapon = 1;
            }

            // quick nade throw
            if (kb.IsKeyDown(Keys.G) && player.Weapons.Count > 1)
            {
                int tmp = player.CurrentWeapon;
                player.CurrentWeapon = 1;
                ShootPlayerGun(player, gameTime);
                player.CurrentWeapon = tmp;
            }

            // check if user is shooting
            if (kb.IsKeyDown(Keys.Space))
            {
                ShootPlayerGun(player, gameTime);
            }

            HandleKeyboardMoveTarget(player, gameTime, kb);

            // check if user is toggling windows/fullscreen modes
            if (kb.IsKeyDown(Keys.F12))
            {
                this.graphics.ToggleFullScreen();
            }
        }

        private void HandleKeyboardMoveTarget(Player player, GameTime gameTime, KeyboardState kb)
        {
            // check to move target up
            float dist = (targetMoveSpeed * gameTime.ElapsedGameTime.Milliseconds);
            Vector2 newPos = Vector2.Zero;

            if (kb.IsKeyDown(Keys.Up) && kb.IsKeyDown(Keys.Left))
            {
                newPos = (Vector2.UnitY + -Vector2.UnitX) * dist;
            }
            else if (kb.IsKeyDown(Keys.Up) && kb.IsKeyDown(Keys.Right))
            {
                newPos = (Vector2.UnitY + Vector2.UnitX) * dist;
            }
            else if (kb.IsKeyDown(Keys.Down) && kb.IsKeyDown(Keys.Left))
            {
                newPos = (-Vector2.UnitY + -Vector2.UnitX) * dist;
            }
            else if (kb.IsKeyDown(Keys.Down) && kb.IsKeyDown(Keys.Right))
            {
                newPos = (-Vector2.UnitY + Vector2.UnitX) * dist;
            }
            else
            {

                if (kb.IsKeyDown(Keys.Up))
                {
                    newPos = Vector2.UnitY * dist;
                }
                else if (kb.IsKeyDown(Keys.Down))
                {
                    newPos = Vector2.UnitY * dist;
                }
                else if (kb.IsKeyDown(Keys.Left))
                {
                    newPos = -Vector2.UnitX * dist;
                }
                else if (kb.IsKeyDown(Keys.Right))
                {
                    newPos = Vector2.UnitX * dist;
                }
            }

            if (newPos != Vector2.Zero)
            {
                player.Target += newPos;
                Mouse.SetPosition((int)player.Target.X, (int)player.Target.Y);
            }
        }


#endif

        private static void ShootPlayerGun(Player player, GameTime gameTime)
        {
            Weapon cw = player.GetCurrentWeapon();
            if (cw != null)
            {
                Vector2 pos = player.Actor.Position;
                FrameSequenceItem f = player.Actor.Frames.GetCurrentFrame();

                // calc direction and distance
                float dir = Helpers.GetRotationInRadians(0, 0, player.BulletSpawnOffsett.X, player.BulletSpawnOffsett.Y);
                float dist = Vector2.Distance(Vector2.Zero, player.BulletSpawnOffsett);

                // position spawn point
                pos.X += (float)Math.Cos(player.Actor.Direction + dir) * dist;
                pos.Y += (float)Math.Sin(player.Actor.Direction + dir) * dist;

                // shoot gun
                cw.Shoot(gameTime, pos, player.Actor.Direction);
            }
        }

        /// <summary>
        /// This is called when the game should draw itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Draw(GameTime gameTime)
        {
            graphics.GraphicsDevice.Clear(Color.CornflowerBlue);

            // draw the target
            DrawTheTarget(p1, gameTime);

            // draw game components
            base.Draw(gameTime);

            // draw info text NOTE Drawing the text here after the base draw call
            // will draw the text on top of everything else
            int xPos = 10;
            int yPos = 10;
            string msg = "Weapon: " + p1.GetCurrentWeapon().Name;
            font.OutputText(msg, xPos, yPos, Color.Black);
            msg = "FPS: " + fps.FPS.ToString() + " Frames: " + fps.Frames.ToString();
            yPos += font.Height + 4;
            font.OutputText(msg, xPos, yPos, Color.Black);

            // draw info text based on current platform
#if Win32
            msg = "W A S D: Moves and strafes";
            yPos += font.Height + 4;
            font.OutputText(msg, xPos, yPos, Color.Black);
            msg = "Up/Down/Left/Right: Moves target";
            yPos += font.Height + 4;
            font.OutputText(msg, xPos, yPos, Color.Black);
            msg = "1 2: Selects weapon, or use mouse wheel";
            yPos += font.Height + 4;
            font.OutputText(msg, xPos, yPos, Color.Black);
            msg = "F12: Toggle full screen";
            yPos += font.Height + 4;
            font.OutputText(msg, xPos, yPos, Color.Black);
            msg = "Spacebar: Shoots";
            yPos += font.Height + 4;
            font.OutputText(msg, xPos, yPos, Color.Black);
            msg = "Use mouse to aim, left click to shoot";
            yPos += font.Height + 4;
            font.OutputText(msg, xPos, yPos, Color.Black);
            msg = "G: throw grenade";
            yPos += font.Height + 4;
            font.OutputText(msg, xPos, yPos, Color.Black);
#else
            msg = "Left Stick moves and strafes";
            yPos += font.Height + 4;
            font.OutputText(msg, xPos, yPos, Color.Black);
            msg = "Left Right Triggers: Selects weapon";
            yPos += font.Height + 4;
            font.OutputText(msg, xPos, yPos, Color.Black);
            msg = "Right Button: Shoots";
            yPos += font.Height + 4;
            font.OutputText(msg, xPos, yPos, Color.Black);
            msg = "Right Stick aims";
            yPos += font.Height + 4;
            font.OutputText(msg, xPos, yPos, Color.Black);
            msg = "Left Button: throw grenade";
            yPos += font.Height + 4;
            font.OutputText(msg, xPos, yPos, Color.Black);
            msg = "Back Button: Quits";
            yPos += font.Height + 4;
            font.OutputText(msg, xPos, yPos, Color.Black);
            msg = "Start Button: Toggles Fullscreen";
            yPos += font.Height + 4;
            font.OutputText(msg, xPos, yPos, Color.Black);
#endif

        }

        private void DrawTheTarget(Player player, GameTime gameTime)
        {
            // get texture and actor target
            Texture2D tex = player.GetCurrentWeapon().TargetTexture;
            Vector2 pos = player.Target;

            // adjust position for texture size
            pos -= new Vector2(tex.Width / 2, tex.Height / 2);

            // draw target
            sb.Begin();
            sb.Draw(tex, pos, Color.Red);
            sb.End();
        }
    }
}