/*
Code written by: Dean Lunz (aka Crated by: X)
Web: http://www.createdbyx.com/ 
 */

#region Using Statements
using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Storage;
#endregion

namespace GunsABlazin
{
    /// <summary>
    /// This is a game component that implements IUpdateable.
    /// </summary>
    public partial class BulletManager : Microsoft.Xna.Framework.DrawableGameComponent
    {


        private FrameSequenceDictionary sequences = new FrameSequenceDictionary();
        private Collections.BulletDictionary bulletReferences = new Collections.BulletDictionary();
        private SpriteBatch sprite;
        private Collections.BulletList bullets = new Collections.BulletList();

        public Collections.BulletList Bullets
        {
            get { return bullets; }
        }

        public Collections.BulletDictionary BulletReferences
        {
            get { return bulletReferences; }
        }

        public FrameSequenceDictionary Sequences
        {
            get { return sequences; }
        }

        public BulletManager(Game game)
            : base(game)
        {
            // TODO: Construct any child components here
        }


        /// <summary>
        /// Allows the game component to perform any initialization it needs to before starting
        /// to run.  This is where it can query for any required services and load content.
        /// </summary>
        public override void Initialize()
        {
            // TODO: Add your initialization code here

            base.Initialize();
            sprite = new SpriteBatch(this.GraphicsDevice);
        }


        /// <summary>
        /// Allows the game component to update itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        public override void Update(GameTime gameTime)
        {
            int idx = 0;
            if (bullets.Count == 0) return;

            // update bullet positions
            while (idx <= bullets.Count - 1)
            {
                // GetHashCode bullet
                Bullet item = bullets[idx];
                // calc nect bullet pos
                Vector2 pos = item.Position;
                pos.X += (float)Math.Cos(item.Direction) * (item.Speed * gameTime.ElapsedGameTime.Milliseconds);
                pos.Y += (float)Math.Sin(item.Direction) * (item.Speed * gameTime.ElapsedGameTime.Milliseconds);

                // check if bullet is still visible
                if (!IsBulletVisible(item))
                {
                    // remove bullet
                    bullets.RemoveAt(idx);
                
                } else if (item.IsMaxDistanceReached == true)
                {
                    // remove bullet if bullet has travel traved as far as it can
                    bullets.RemoveAt(idx);

                } else {
                    // set bullets new pos
                    item.Position = pos;
                    idx++;
                }
            }

            base.Update(gameTime);
        }

        bool IsBulletVisible(Bullet b)
        {
            if (b.Position.X < 0) return false;
            if (b.Position.Y < 0) return false;
            if (b.Position.X > this.GraphicsDevice.Viewport.Width) return false;
            if (b.Position.Y > this.GraphicsDevice.Viewport.Height) return false;
            return true;
        }

        /// <summary>
        /// Allows the game component to draw itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        public override void Draw(GameTime gameTime)
        {
            // draw the bullets on the screen
            foreach (Bullet item in bullets)
            {
                sprite.Begin();
                FrameSequenceItem f;
                f = item.BulletSequence.GetCurrentFrame();
                sprite.Draw(f.Texture, new Vector2(item.Position.X, item.Position.Y), null, Color.White,
                            item.Direction + MathHelper.ToRadians(90), f.Center, f.Scale, SpriteEffects.None, 0);
                sprite.End();
            }

            base.Draw(gameTime);
        }

    }
}


